@@ -86,7 +86,6 @@ import java.util.Optional;
 import java.util.Set;
 import java.util.function.Predicate;
 import javax.lang.model.element.AnnotationMirror;
-import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.TypeVariable;
 import org.checkerframework.errorprone.dataflow.analysis.Analysis;
 import org.checkerframework.errorprone.dataflow.analysis.ForwardAnalysisImpl;
@@ -468,12 +467,12 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
     Nullness value = inputs.valueOfSubNode(node.getExpression());
 
     Node target = node.getTarget();
-    if (target instanceof LocalVariableNode) {
-      updates.set((LocalVariableNode) target, value);
+    if (target instanceof LocalVariableNode localVariableNode) {
+      updates.set(localVariableNode, value);
     }
 
-    if (target instanceof ArrayAccessNode) {
-      setNonnullIfTrackable(updates, ((ArrayAccessNode) target).getArray());
+    if (target instanceof ArrayAccessNode arrayAccessNode) {
+      setNonnullIfTrackable(updates, arrayAccessNode.getArray());
     }
 
     if (target instanceof FieldAccessNode fieldAccess) {
@@ -499,7 +498,7 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
      * It's not clear to me that this is technically correct, but it works in practice with the
      * bytecode generated by both javac and ecj.
      *
-     * http://stackoverflow.com/q/12850676/28465
+     * https://stackoverflow.com/q/12850676/28465
      */
     return value;
   }
@@ -672,25 +671,21 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
   }
 
   private static boolean hasNonNullConstantValue(LocalVariableNode node) {
-    if (node.getElement() instanceof VariableElement) {
-      VariableElement element = (VariableElement) node.getElement();
-      return (element.getConstantValue() != null);
-    }
-    return false;
+    return node.getElement() != null && node.getElement().getConstantValue() != null;
   }
 
   private static @Nullable ClassAndField tryGetFieldSymbol(Tree tree) {
     Symbol symbol = tryGetSymbol(tree);
-    if (symbol instanceof VarSymbol) {
-      return ClassAndField.make((VarSymbol) symbol);
+    if (symbol instanceof VarSymbol varSymbol) {
+      return ClassAndField.make(varSymbol);
     }
     return null;
   }
 
   static @Nullable ClassAndMethod tryGetMethodSymbol(MethodInvocationTree tree, Types types) {
     Symbol symbol = tryGetSymbol(tree.getMethodSelect());
-    if (symbol instanceof MethodSymbol) {
-      return ClassAndMethod.make((MethodSymbol) symbol, types);
+    if (symbol instanceof MethodSymbol methodSymbol) {
+      return ClassAndMethod.make(methodSymbol, types);
     }
     return null;
   }
@@ -700,14 +695,14 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
    * back on core.
    */
   private static @Nullable Symbol tryGetSymbol(Tree tree) {
-    if (tree instanceof JCIdent) {
-      return ((JCIdent) tree).sym;
+    if (tree instanceof JCIdent jCIdent) {
+      return jCIdent.sym;
     }
-    if (tree instanceof JCFieldAccess) {
-      return ((JCFieldAccess) tree).sym;
+    if (tree instanceof JCFieldAccess jCFieldAccess) {
+      return jCFieldAccess.sym;
     }
-    if (tree instanceof JCVariableDecl) {
-      return ((JCVariableDecl) tree).sym;
+    if (tree instanceof JCVariableDecl jCVariableDecl) {
+      return jCVariableDecl.sym;
     }
     return null;
   }
@@ -763,8 +758,8 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
     Optional<Nullness> declaredNullness = NullnessAnnotations.fromAnnotationsOn(accessed.symbol);
     if (declaredNullness.isEmpty()) {
       Type ftype = accessed.symbol.type;
-      if (ftype instanceof TypeVariable) {
-        declaredNullness = NullnessAnnotations.getUpperBound((TypeVariable) ftype);
+      if (ftype instanceof TypeVariable typeVariable) {
+        declaredNullness = NullnessAnnotations.getUpperBound(typeVariable);
       } else {
         declaredNullness = NullnessAnnotations.fromDefaultAnnotations(accessed.symbol);
       }
@@ -810,11 +805,11 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
       // missing types.
       if (fieldDeclPath == null
           || fieldDeclPath.getCompilationUnit() != compilationUnit
-          || !(fieldDeclPath.getLeaf() instanceof VariableTree)) {
+          || !(fieldDeclPath.getLeaf() instanceof VariableTree variableTree)) {
         return null;
       }
 
-      ExpressionTree initializer = ((VariableTree) fieldDeclPath.getLeaf()).getInitializer();
+      ExpressionTree initializer = variableTree.getInitializer();
       if (initializer == null) {
         return null;
       }
@@ -843,12 +838,12 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
   }
 
   private static void setNonnullIfTrackable(Updates updates, Node node) {
-    if (node instanceof LocalVariableNode) {
-      updates.set((LocalVariableNode) node, NONNULL);
-    } else if (node instanceof FieldAccessNode) {
-      updates.set((FieldAccessNode) node, NONNULL);
-    } else if (node instanceof VariableDeclarationNode) {
-      updates.set((VariableDeclarationNode) node, NONNULL);
+    if (node instanceof LocalVariableNode localVariableNode) {
+      updates.set(localVariableNode, NONNULL);
+    } else if (node instanceof FieldAccessNode fieldAccessNode) {
+      updates.set(fieldAccessNode, NONNULL);
+    } else if (node instanceof VariableDeclarationNode variableDeclarationNode) {
+      updates.set(variableDeclarationNode, NONNULL);
     }
   }
 
@@ -903,14 +898,14 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
       MemberName calleeName, List<Node> arguments, Types types, Symtab symtab) {
     // we don't care about class name -- we're matching against Object.equals(Object)
     // this implies that non-overriding methods are assumed to be null-guaranteeing.
-    // Also see http://errorprone.info/bugpattern/NonOverridingEquals
+    // Also see https://errorprone.info/bugpattern/NonOverridingEquals
     if (!calleeName.member.equals("equals") || arguments.size() != 1) {
       return false;
     }
-    if (!(getOnlyElement(arguments).getTree() instanceof JCIdent)) {
+    if (!(getOnlyElement(arguments).getTree() instanceof JCIdent jCIdent)) {
       return false;
     }
-    Symbol sym = ((JCIdent) getOnlyElement(arguments).getTree()).sym;
+    Symbol sym = jCIdent.sym;
     if (sym == null || sym.type == null) {
       return false;
     }
@@ -928,8 +923,8 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
       // TODO(cpovirk): better handling of varargs
       if (i >= 0 && i < arguments.size()) {
         Node argument = arguments.get(i);
-        if (argument instanceof LocalVariableNode) {
-          result.add((LocalVariableNode) argument);
+        if (argument instanceof LocalVariableNode localVariableNode) {
+          result.add(localVariableNode);
         }
       }
     }
