@@ -37,7 +37,6 @@ import static javax.lang.model.element.Modifier.STATIC;
 import com.google.common.base.VerifyException;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableSet;
-import com.google.common.collect.Sets;
 import com.google.errorprone.VisitorState;
 import com.google.errorprone.dataflow.nullnesspropagation.Nullness;
 import com.google.errorprone.matchers.ChildMultiMatcher.MatchType;
@@ -62,6 +61,7 @@ import com.sun.source.tree.EnhancedForLoopTree;
 import com.sun.source.tree.ExpressionStatementTree;
 import com.sun.source.tree.ExpressionTree;
 import com.sun.source.tree.IdentifierTree;
+import com.sun.source.tree.LambdaExpressionTree;
 import com.sun.source.tree.LiteralTree;
 import com.sun.source.tree.MemberSelectTree;
 import com.sun.source.tree.MethodInvocationTree;
@@ -366,11 +366,8 @@ public class Matchers {
   public static Matcher<ExpressionTree> methodInvocation(
       Matcher<ExpressionTree> methodSelectMatcher) {
     return (expressionTree, state) -> {
-      if (!(expressionTree instanceof MethodInvocationTree)) {
-        return false;
-      }
-      MethodInvocationTree tree = (MethodInvocationTree) expressionTree;
-      return methodSelectMatcher.matches(tree.getMethodSelect(), state);
+      return expressionTree instanceof MethodInvocationTree tree
+          && methodSelectMatcher.matches(tree.getMethodSelect(), state);
     };
   }
 
@@ -661,8 +658,7 @@ public class Matchers {
 
   public static Matcher<ExpressionTree> classLiteral(Matcher<? super ExpressionTree> classMatcher) {
     return (tree, state) -> {
-      if (tree.getKind() == Kind.MEMBER_SELECT) {
-        MemberSelectTree select = (MemberSelectTree) tree;
+      if (tree instanceof MemberSelectTree select) {
         return select.getIdentifier().contentEquals("class")
             && classMatcher.matches(select.getExpression(), state);
       }
@@ -954,7 +950,7 @@ public class Matchers {
   public static Matcher<ClassTree> hasMethod(Matcher<MethodTree> methodMatcher) {
     return (t, state) -> {
       for (Tree member : t.getMembers()) {
-        if (member instanceof MethodTree && methodMatcher.matches((MethodTree) member, state)) {
+        if (member instanceof MethodTree methodTree && methodMatcher.matches(methodTree, state)) {
           return true;
         }
       }
@@ -997,6 +993,11 @@ public class Matchers {
     return (classTree, state) -> getSymbol(classTree).getKind() == ElementKind.ENUM;
   }
 
+  /** Matches if a {@link ClassTree} is a {@code record} declaration. */
+  public static Matcher<ClassTree> isRecord() {
+    return (classTree, state) -> getSymbol(classTree).getKind() == ElementKind.RECORD;
+  }
+
   /**
    * Matches an class based on whether it is nested in another class or method.
    *
@@ -1087,8 +1088,8 @@ public class Matchers {
   /** Matches an {@link ExpressionStatementTree} based on its {@link ExpressionTree}. */
   public static Matcher<StatementTree> expressionStatement(Matcher<ExpressionTree> matcher) {
     return (statementTree, state) ->
-        statementTree instanceof ExpressionStatementTree
-            && matcher.matches(((ExpressionStatementTree) statementTree).getExpression(), state);
+        statementTree instanceof ExpressionStatementTree expressionStatementTree
+            && matcher.matches(expressionStatementTree.getExpression(), state);
   }
 
   static Matcher<Tree> isSymbol(java.lang.Class<? extends Symbol> symbolClass) {
@@ -1312,18 +1313,11 @@ public class Matchers {
     return anyOf(matchers);
   }
 
-  private static final ImmutableSet<Kind> DECLARATION =
-      Sets.immutableEnumSet(Kind.LAMBDA_EXPRESSION, Kind.CLASS, Kind.ENUM, Kind.INTERFACE);
-
-  private static boolean isDeclaration(Kind kind) {
-    return DECLARATION.contains(kind) || kind.name().equals("RECORD");
-  }
-
   public static boolean methodCallInDeclarationOfThrowingRunnable(VisitorState state) {
     return stream(state.getPath())
         // Find the nearest definitional context for this method invocation
         // (i.e.: the nearest surrounding class or lambda)
-        .filter(t -> isDeclaration(t.getKind()))
+        .filter(t -> t instanceof LambdaExpressionTree || t instanceof ClassTree)
         .findFirst()
         .map(t -> isThrowingFunctionalInterface(getType(t), state))
         .orElseThrow(VerifyException::new);
@@ -1443,7 +1437,7 @@ public class Matchers {
     return (Matcher<T>) STATIC_EQUALS;
   }
 
-  private static final Matcher<ExpressionTree> INSTANCE_EQUALS =
+  public static final Matcher<ExpressionTree> INSTANCE_EQUALS =
       allOf(
           instanceMethod().anyClass().named("equals").withParameters("java.lang.Object"),
           Matchers::methodReturnsBoolean);
@@ -1581,10 +1575,10 @@ public class Matchers {
   public static Matcher<StatementTree> matchExpressionReturn(
       Matcher<ExpressionTree> expressionTreeMatcher) {
     return (statement, state) -> {
-      if (!(statement instanceof ReturnTree)) {
+      if (!(statement instanceof ReturnTree returnTree)) {
         return false;
       }
-      ExpressionTree expression = ((ReturnTree) statement).getExpression();
+      ExpressionTree expression = returnTree.getExpression();
       if (expression == null) {
         return false;
       }
