@@ -160,6 +160,20 @@ public final class StatementSwitchToExpressionSwitch extends BugChecker
     SOME_OR_ALL_CASES_DONT_QUALIFY
   }
 
+  /**
+   * The kind of null/default cases included within a single CaseTree.
+   *
+   * <p>This enum is used to classify whether a CaseTree includes a null and/or default. Referencing
+   * JLS 21 §14.11.1, the `SwitchLabel:` production has specific rules applicable to null/default
+   * cases: `case null, [default]` and `default`. All other scenarios are lumped into KIND_NEITHER.
+   */
+  enum NullDefaultKind {
+    KIND_NULL_AND_DEFAULT,
+    KIND_DEFAULT,
+    KIND_NULL,
+    KIND_NEITHER
+  }
+
   private final boolean enableDirectConversion;
   private final boolean enableReturnSwitchConversion;
   private final boolean enableAssignmentSwitchConversion;
@@ -293,12 +307,15 @@ public final class StatementSwitchToExpressionSwitch extends BugChecker
         // Case patterns are not currently supported by the checker.
         return DEFAULT_ANALYSIS_RESULT;
       }
-      boolean isDefaultCase = caseTree.getExpressions().isEmpty();
+
+      NullDefaultKind nullDefaultKind = analyzeCaseForNullAndDefault(caseTree);
+      boolean isDefaultCase =
+          nullDefaultKind.equals(NullDefaultKind.KIND_DEFAULT)
+              || nullDefaultKind.equals(NullDefaultKind.KIND_NULL_AND_DEFAULT);
       isNullCase.set(
           caseIndex,
-          !isDefaultCase
-              && caseTree.getExpressions().stream()
-                  .anyMatch(expressionTree -> expressionTree.getKind() == Kind.NULL_LITERAL));
+          nullDefaultKind.equals(NullDefaultKind.KIND_NULL)
+              || nullDefaultKind.equals(NullDefaultKind.KIND_NULL_AND_DEFAULT));
       hasDefaultCase |= isDefaultCase;
 
       // Null case can never be grouped with a preceding case
@@ -374,9 +391,26 @@ public final class StatementSwitchToExpressionSwitch extends BugChecker
 
       // Find any symbols referenced in this case that were defined in a previous case, and thus
       // should be hoisted out of the switch block
-      symbolsDefinedInPreviousCases.keySet().stream()
-          .filter(symbol -> hasReadsOrWritesOfVariableInTree(symbol, caseTree))
-          .forEach(symbol -> symbolsToHoist.put(symbol, symbolsDefinedInPreviousCases.get(symbol)));
+      ImmutableSet<VarSymbol> newSymbolsToHoist =
+          symbolsDefinedInPreviousCases.keySet().stream()
+              .filter(symbol -> hasReadsOrWritesOfVariableInTree(symbol, caseTree))
+              .collect(toImmutableSet());
+
+      // Ensure that hoisting does not conflict with other declared variables in switch scope
+      boolean hasNamingConflict =
+          newSymbolsToHoist.stream()
+              // In principle, this search could be terminated after checking up through `caseTree`
+              // (inclusive) because naming conflicts after that would have caused compile-time
+              // errors.  For simplicity, the search is not restricted.
+              .filter(symbol -> declaresAnotherVariableNamed(symbol, switchTree))
+              .findAny()
+              .isPresent();
+      if (hasNamingConflict) {
+        return DEFAULT_ANALYSIS_RESULT;
+      }
+
+      newSymbolsToHoist.forEach(
+          symbol -> symbolsToHoist.put(symbol, symbolsDefinedInPreviousCases.get(symbol)));
 
       // Analyze for return switch and assignment switch conversion
       returnSwitchCaseQualifications =
@@ -547,6 +581,32 @@ public final class StatementSwitchToExpressionSwitch extends BugChecker
     return referencedLocalVariables.contains(symbol);
   }
 
+  /**
+   * Determines whether the switch statement has a case that declares a local variable with the same
+   * name as the supplied {@code symbol}.
+   */
+  private static boolean declaresAnotherVariableNamed(VarSymbol symbol, SwitchTree switchTree) {
+    return new TreeScanner<Boolean, Void>() {
+      @Override
+      public Boolean visitVariable(VariableTree variableTree, Void unused) {
+        // If the variable is named the same as the symbol, but it's not the original declaration
+        // of the symbol, then there's a name conflict.
+        if (variableTree.getName().contentEquals(symbol.name.toString())) {
+          VarSymbol thisVarSymbol = ASTHelpers.getSymbol(variableTree);
+          if (!thisVarSymbol.equals(symbol)) {
+            return true;
+          }
+        }
+        return super.visitVariable(variableTree, null);
+      }
+
+      @Override
+      public Boolean reduce(@Nullable Boolean left, @Nullable Boolean right) {
+        return Objects.equals(left, true) || Objects.equals(right, true);
+      }
+    }.scan(switchTree, null);
+  }
+
   /**
    * Renders the Java source code for a [compound] assignment operator. The parameter must be either
    * an {@code AssignmentTree} or a {@code CompoundAssignmentTree}.
@@ -563,6 +623,21 @@ public final class StatementSwitchToExpressionSwitch extends BugChecker
     return pretty.operatorName(jcAssignOp.getTag().noAssignOp()) + EQUALS_STRING;
   }
 
+  /**
+   * Renders the Java source prefix needed for the supplied {@code nullDefaultKind}, incorporating
+   * whether the `default` case should be removed.
+   */
+  private static String renderNullDefaultKindPrefix(
+      NullDefaultKind nullDefaultKind, boolean removeDefault) {
+
+    return switch (nullDefaultKind) {
+      case KIND_NULL_AND_DEFAULT -> removeDefault ? "case null" : "case null, default";
+      case KIND_NULL -> "case null";
+      case KIND_DEFAULT -> removeDefault ? "" : "default";
+      case KIND_NEITHER -> "case ";
+    };
+  }
+
   /**
    * Analyze a single {@code case} of a single {@code switch} statement to determine whether it is
    * convertible to a return switch. The supplied {@code previousCaseQualifications} is updated and
@@ -908,10 +983,10 @@ public final class StatementSwitchToExpressionSwitch extends BugChecker
     boolean firstCaseInGroup = true;
     for (int caseIndex = 0; caseIndex < cases.size(); caseIndex++) {
       CaseTree caseTree = cases.get(caseIndex);
-      boolean isDefaultCase = isSwitchDefault(caseTree);
+      NullDefaultKind nullDefaultKind = analyzeCaseForNullAndDefault(caseTree);
 
-      if (removeDefault && isDefaultCase) {
-        // Skip default case
+      if (removeDefault && nullDefaultKind.equals(NullDefaultKind.KIND_DEFAULT)) {
+        // Skip removed default (and its code) entirely
         continue;
       }
 
@@ -928,14 +1003,19 @@ public final class StatementSwitchToExpressionSwitch extends BugChecker
                     ? extractCommentsBeforeFirstCase(switchTree, allSwitchComments).orElse("")
                     : "");
 
-        replacementCodeBuilder.append("\n  ");
-        if (!isDefaultCase) {
-          replacementCodeBuilder.append("case ");
+        replacementCodeBuilder
+            .append("\n  ")
+            .append(renderNullDefaultKindPrefix(nullDefaultKind, removeDefault));
+      } else {
+        // Second or later case in our group
+        if (nullDefaultKind.equals(NullDefaultKind.KIND_DEFAULT)) {
+          replacementCodeBuilder.append("default");
         }
       }
-      replacementCodeBuilder.append(
-          isDefaultCase ? "default" : printCaseExpressions(caseTree, state));
 
+      if (nullDefaultKind.equals(NullDefaultKind.KIND_NEITHER)) {
+        replacementCodeBuilder.append(printCaseExpressions(caseTree, state));
+      }
       Optional<String> commentsAfterCaseOptional =
           extractCommentsAfterCase(switchTree, allSwitchComments, state, caseIndex);
       if (analysisResult.groupedWithNextCase().get(caseIndex)) {
@@ -1052,9 +1132,9 @@ public final class StatementSwitchToExpressionSwitch extends BugChecker
     boolean firstCaseInGroup = true;
     for (int caseIndex = 0; caseIndex < cases.size(); caseIndex++) {
       CaseTree caseTree = cases.get(caseIndex);
-      boolean isDefaultCase = isSwitchDefault(caseTree);
-      if (removeDefault && isDefaultCase) {
-        // Skip default case
+      NullDefaultKind nullDefaultKind = analyzeCaseForNullAndDefault(caseTree);
+      if (removeDefault && nullDefaultKind.equals(NullDefaultKind.KIND_DEFAULT)) {
+        // Skip removed default (and its code) entirely
         continue;
       }
 
@@ -1068,13 +1148,19 @@ public final class StatementSwitchToExpressionSwitch extends BugChecker
                     ? extractCommentsBeforeFirstCase(switchTree, allSwitchComments).orElse("")
                     : "");
 
-        replacementCodeBuilder.append("\n  ");
-        if (!isDefaultCase) {
-          replacementCodeBuilder.append("case ");
+        replacementCodeBuilder
+            .append("\n  ")
+            .append(renderNullDefaultKindPrefix(nullDefaultKind, removeDefault));
+      } else {
+        // Second or later case in our group
+        if (nullDefaultKind.equals(NullDefaultKind.KIND_DEFAULT)) {
+          replacementCodeBuilder.append("default");
         }
       }
-      replacementCodeBuilder.append(
-          isDefaultCase ? "default" : printCaseExpressions(caseTree, state));
+
+      if (nullDefaultKind.equals(NullDefaultKind.KIND_NEITHER)) {
+        replacementCodeBuilder.append(printCaseExpressions(caseTree, state));
+      }
 
       Optional<String> commentsAfterCaseOptional =
           extractCommentsAfterCase(switchTree, allSwitchComments, state, caseIndex);
@@ -1280,9 +1366,10 @@ public final class StatementSwitchToExpressionSwitch extends BugChecker
     boolean firstCaseInGroup = true;
     for (int caseIndex = 0; caseIndex < cases.size(); caseIndex++) {
       CaseTree caseTree = cases.get(caseIndex);
-      boolean isDefaultCase = isSwitchDefault(caseTree);
-      if (removeDefault && isDefaultCase) {
-        // Remove `default:` case (and its code, if any) from the SuggestedFix
+      NullDefaultKind nullDefaultKind = analyzeCaseForNullAndDefault(caseTree);
+
+      if (removeDefault && nullDefaultKind.equals(NullDefaultKind.KIND_DEFAULT)) {
+        // Skip removed default (and its code) entirely
         continue;
       }
       ImmutableList<StatementTree> filteredStatements = filterOutRedundantBreak(caseTree);
@@ -1297,13 +1384,19 @@ public final class StatementSwitchToExpressionSwitch extends BugChecker
                     ? extractCommentsBeforeFirstCase(switchTree, allSwitchComments).orElse("")
                     : "");
 
-        replacementCodeBuilder.append("\n  ");
-        if (!isDefaultCase) {
-          replacementCodeBuilder.append("case ");
+        replacementCodeBuilder
+            .append("\n  ")
+            .append(renderNullDefaultKindPrefix(nullDefaultKind, removeDefault));
+      } else {
+        // Second or later case in our group
+        if (nullDefaultKind.equals(NullDefaultKind.KIND_DEFAULT)) {
+          replacementCodeBuilder.append("default");
         }
       }
-      replacementCodeBuilder.append(
-          isDefaultCase ? "default" : printCaseExpressions(caseTree, state));
+
+      if (nullDefaultKind.equals(NullDefaultKind.KIND_NEITHER)) {
+        replacementCodeBuilder.append(printCaseExpressions(caseTree, state));
+      }
 
       Optional<String> commentsAfterCaseOptional =
           extractCommentsAfterCase(switchTree, allSwitchComments, state, caseIndex);
@@ -1735,6 +1828,26 @@ public final class StatementSwitchToExpressionSwitch extends BugChecker
     return transformedBlockBuilder.toString();
   }
 
+  /**
+   * Determines whether the supplied {@code caseTree} case contains `case null` and/or `default`.
+   */
+  private static NullDefaultKind analyzeCaseForNullAndDefault(CaseTree caseTree) {
+    boolean hasDefault = isSwitchDefault(caseTree);
+    boolean hasNull =
+        caseTree.getExpressions().stream()
+            .anyMatch(expression -> expression.getKind().equals(Tree.Kind.NULL_LITERAL));
+
+    if (hasNull && hasDefault) {
+      return NullDefaultKind.KIND_NULL_AND_DEFAULT;
+    } else if (hasNull) {
+      return NullDefaultKind.KIND_NULL;
+    } else if (hasDefault) {
+      return NullDefaultKind.KIND_DEFAULT;
+    }
+
+    return NullDefaultKind.KIND_NEITHER;
+  }
+
   @AutoValue
   abstract static class AnalysisResult {
     /** Whether the statement switch can be directly converted to an expression switch */
