@@ -34,14 +34,12 @@ import static com.google.errorprone.util.ASTHelpers.hasExplicitSource;
 import static com.google.errorprone.util.ASTHelpers.isAbstract;
 import static com.google.errorprone.util.ASTHelpers.isStatic;
 import static com.google.errorprone.util.ASTHelpers.isSubtype;
-import static com.google.errorprone.util.ASTHelpers.shouldKeep;
 import static com.google.errorprone.util.SideEffectAnalysis.hasSideEffect;
 import static com.sun.source.tree.Tree.Kind.POSTFIX_DECREMENT;
 import static com.sun.source.tree.Tree.Kind.POSTFIX_INCREMENT;
 import static com.sun.source.tree.Tree.Kind.PREFIX_DECREMENT;
 import static com.sun.source.tree.Tree.Kind.PREFIX_INCREMENT;
 
-import com.google.auto.value.AutoValue;
 import com.google.common.base.Ascii;
 import com.google.common.collect.ArrayListMultimap;
 import com.google.common.collect.ImmutableList;
@@ -66,9 +64,9 @@ import com.google.errorprone.matchers.Description;
 import com.google.errorprone.suppliers.Supplier;
 import com.google.errorprone.suppliers.Suppliers;
 import com.google.errorprone.util.ASTHelpers;
-import com.sun.source.tree.AnnotationTree;
 import com.sun.source.tree.ArrayAccessTree;
 import com.sun.source.tree.AssignmentTree;
+import com.sun.source.tree.BindingPatternTree;
 import com.sun.source.tree.ClassTree;
 import com.sun.source.tree.CompilationUnitTree;
 import com.sun.source.tree.CompoundAssignmentTree;
@@ -80,6 +78,7 @@ import com.sun.source.tree.ExpressionTree;
 import com.sun.source.tree.ForLoopTree;
 import com.sun.source.tree.IdentifierTree;
 import com.sun.source.tree.IfTree;
+import com.sun.source.tree.InstanceOfTree;
 import com.sun.source.tree.LambdaExpressionTree;
 import com.sun.source.tree.MemberReferenceTree;
 import com.sun.source.tree.MemberSelectTree;
@@ -88,7 +87,6 @@ import com.sun.source.tree.MethodTree;
 import com.sun.source.tree.ReturnTree;
 import com.sun.source.tree.StatementTree;
 import com.sun.source.tree.Tree;
-import com.sun.source.tree.Tree.Kind;
 import com.sun.source.tree.TryTree;
 import com.sun.source.tree.UnaryTree;
 import com.sun.source.tree.VariableTree;
@@ -100,7 +98,6 @@ import com.sun.source.util.TreeScanner;
 import com.sun.tools.javac.code.Symbol;
 import com.sun.tools.javac.code.Symbol.ClassSymbol;
 import com.sun.tools.javac.code.Symbol.MethodSymbol;
-import com.sun.tools.javac.code.Symbol.TypeSymbol;
 import com.sun.tools.javac.code.Symbol.VarSymbol;
 import com.sun.tools.javac.code.Type;
 import com.sun.tools.javac.tree.JCTree;
@@ -135,29 +132,6 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
 
   private final ImmutableSet<String> exemptNames;
 
-  /**
-   * The set of annotation full names which exempt annotated element from being reported as unused.
-   *
-   * <p>Try to avoid adding more annotations here. Annotating these annotations with {@code @Keep}
-   * has the same effect; this list is chiefly for third-party annotations which cannot be
-   * annotated.
-   */
-  private static final ImmutableSet<String> EXEMPTING_VARIABLE_ANNOTATIONS =
-      ImmutableSet.of(
-          "javax.persistence.Basic",
-          "javax.persistence.Column",
-          "javax.persistence.Id",
-          "javax.persistence.Version",
-          "javax.xml.bind.annotation.XmlElement",
-          "org.junit.Rule",
-          "org.junit.jupiter.api.extension.RegisterExtension",
-          "org.openqa.selenium.support.FindAll",
-          "org.openqa.selenium.support.FindBy",
-          "org.openqa.selenium.support.FindBys",
-          "org.apache.beam.sdk.transforms.DoFn.TimerId",
-          "org.apache.beam.sdk.transforms.DoFn.StateId",
-          "org.springframework.boot.test.mock.mockito.MockBean");
-
   // TODO(ghm): Find a sensible place to dedupe this with UnnecessarilyVisible.
   private static final ImmutableSet<String> ANNOTATIONS_INDICATING_PARAMETERS_SHOULD_BE_CHECKED =
       ImmutableSet.of(
@@ -168,7 +142,11 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
           "com.google.inject.multibindings.ProvidesIntoMap",
           "com.google.inject.multibindings.ProvidesIntoSet",
           "dagger.Provides",
-          "javax.inject.Inject");
+          "jakarta.inject.Inject",
+          "javax.inject.Inject",
+          // Parameters on test methods imply the test is parameterised, and those parameters should
+          // be used or removed.
+          "org.junit.Test");
 
   private final ImmutableSet<String> methodAnnotationsExemptingParameters;
 
@@ -190,10 +168,13 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
 
   private final boolean reportInjectedFields;
 
+  private final WellKnownKeep wellKnownKeep;
+
   @Inject
-  UnusedVariable(ErrorProneFlags flags) {
+  UnusedVariable(ErrorProneFlags flags, WellKnownKeep wellKnownKeep) {
     this.methodAnnotationsExemptingParameters =
         ImmutableSet.<String>builder()
+            .add("com.google.common.eventbus.Subscribe")
             .add("org.robolectric.annotation.Implementation")
             .addAll(flags.getListOrEmpty("Unused:methodAnnotationsExemptingParameters"))
             .build();
@@ -211,6 +192,8 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
             .add("unused")
             .addAll(flags.getSetOrEmpty("Unused:exemptPrefixes"))
             .build();
+
+    this.wellKnownKeep = wellKnownKeep;
   }
 
   @Override
@@ -325,10 +308,9 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
                     allUsageSites.stream()
                         .noneMatch(
                             tp ->
-                                tp.getLeaf() instanceof ExpressionStatementTree
-                                    && ((ExpressionStatementTree) tp.getLeaf())
-                                        .getExpression()
-                                        .equals(a)))
+                                tp.getLeaf()
+                                        instanceof ExpressionStatementTree expressionStatementTree
+                                    && expressionStatementTree.getExpression().equals(a)))
             .findFirst();
     if (removedVariableTree.isPresent() && reassignment.isPresent()) {
       return SuggestedFix.prefixWith( // not needed if top-level statement
@@ -342,6 +324,7 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
       case FIELD -> "field";
       case LOCAL_VARIABLE -> "local variable";
       case PARAMETER -> "parameter";
+      case BINDING_VARIABLE -> "binding variable";
       default -> "variable";
     };
   }
@@ -435,26 +418,28 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
         SuggestedFix.builder().setShortDescription("remove unused variable and any side effects");
     for (TreePath usagePath : usagePaths) {
       StatementTree statement = (StatementTree) usagePath.getLeaf();
-      if (statement.getKind() == Kind.VARIABLE) {
+      if (statement instanceof VariableTree variableTree) {
         if (getSymbol(statement).getKind() == ElementKind.PARAMETER) {
           continue;
         }
-        VariableTree variableTree = (VariableTree) statement;
         ExpressionTree initializer = variableTree.getInitializer();
         if (hasSideEffect(initializer) && TOP_LEVEL_EXPRESSIONS.contains(initializer.getKind())) {
           encounteredSideEffects = true;
-          if (varKind == ElementKind.FIELD) {
-            String newContent =
-                String.format(
-                    "%s{ %s; }",
-                    isStatic(varSymbol) ? "static " : "", state.getSourceForNode(initializer));
-            keepSideEffectsFix.merge(
-                SuggestedFixes.replaceIncludingComments(usagePath, newContent, state));
-            removeSideEffectsFix.replace(statement, "");
-          } else {
-            keepSideEffectsFix.replace(
-                statement, String.format("%s;", state.getSourceForNode(initializer)));
-            removeSideEffectsFix.replace(statement, "");
+          switch (varKind) {
+            case FIELD -> {
+              String newContent =
+                  String.format(
+                      "%s{ %s; }",
+                      isStatic(varSymbol) ? "static " : "", state.getSourceForNode(initializer));
+              keepSideEffectsFix.merge(
+                  SuggestedFixes.replaceIncludingComments(usagePath, newContent, state));
+              removeSideEffectsFix.replace(statement, "");
+            }
+            default -> {
+              keepSideEffectsFix.replace(
+                  statement, String.format("%s;", state.getSourceForNode(initializer)));
+              removeSideEffectsFix.replace(statement, "");
+            }
           }
         } else if (isEnhancedForLoopVar(usagePath)) {
           String modifiers =
@@ -471,6 +456,10 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
           // fix in this case because we can't just remove the enhanced for loop variable.
           keepSideEffectsFix.replace(variableTree, newContent);
           removeSideEffectsFix.replace(variableTree, newContent);
+        } else if (varKind == ElementKind.BINDING_VARIABLE) {
+          removeSideEffectsFix.replace(
+              variableTree, state.getSourceForNode(variableTree.getType()));
+          keepSideEffectsFix.replace(variableTree, state.getSourceForNode(variableTree.getType()));
         } else {
           String replacement = needsBlock(usagePath) ? "{}" : "";
           keepSideEffectsFix.merge(
@@ -479,11 +468,11 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
               SuggestedFixes.replaceIncludingComments(usagePath, replacement, state));
         }
         continue;
-      } else if (statement.getKind() == Kind.EXPRESSION_STATEMENT) {
-        JCTree tree = (JCTree) ((ExpressionStatementTree) statement).getExpression();
+      } else if (statement instanceof ExpressionStatementTree expressionStatementTree) {
+        JCTree tree = (JCTree) expressionStatementTree.getExpression();
 
-        if (tree instanceof CompoundAssignmentTree) {
-          if (hasSideEffect(((CompoundAssignmentTree) tree).getExpression())) {
+        if (tree instanceof CompoundAssignmentTree compoundAssignmentTree) {
+          if (hasSideEffect(compoundAssignmentTree.getExpression())) {
             // If it's a compound assignment, there's no reason we'd want to remove the expression,
             // so don't set `encounteredSideEffects` based on this usage.
             SuggestedFix replacement =
@@ -495,8 +484,8 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
             removeSideEffectsFix.merge(replacement);
             continue;
           }
-        } else if (tree instanceof AssignmentTree) {
-          if (hasSideEffect(((AssignmentTree) tree).getExpression())) {
+        } else if (tree instanceof AssignmentTree assignmentTree) {
+          if (hasSideEffect(assignmentTree.getExpression())) {
             encounteredSideEffects = true;
             keepSideEffectsFix.replace(
                 tree.getStartPosition(), ((JCAssign) tree).getExpression().getStartPosition(), "");
@@ -516,13 +505,12 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
 
   private static ImmutableList<SuggestedFix> buildUnusedParameterFixes(
       Symbol varSymbol, List<TreePath> usagePaths, VisitorState state) {
-    if (!(varSymbol.owner instanceof MethodSymbol)
-        || !((MethodSymbol) varSymbol.owner).params().contains(varSymbol)
+    if (!(varSymbol.owner instanceof MethodSymbol methodSymbol)
+        || !methodSymbol.params().contains(varSymbol)
         || !canBeRemoved(varSymbol.owner, state)) {
       // We're presumably in a lambda. Don't try to generate a fix.
       return ImmutableList.of();
     }
-    MethodSymbol methodSymbol = (MethodSymbol) varSymbol.owner;
     int index = methodSymbol.params.indexOf(varSymbol);
     RangeSet<Integer> deletions = TreeRangeSet.create();
     for (TreePath path : usagePaths) {
@@ -587,26 +575,8 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
   private static boolean isEnhancedForLoopVar(TreePath variablePath) {
     Tree tree = variablePath.getLeaf();
     Tree parent = variablePath.getParentPath().getLeaf();
-    return parent instanceof EnhancedForLoopTree
-        && ((EnhancedForLoopTree) parent).getVariable() == tree;
-  }
-
-  /**
-   * Looks at the list of {@code annotations} and see if there is any annotation which exists {@code
-   * exemptingAnnotations}.
-   */
-  private static boolean exemptedByAnnotation(List<? extends AnnotationTree> annotations) {
-    for (AnnotationTree annotation : annotations) {
-      Type annotationType = ASTHelpers.getType(annotation);
-      if (annotationType == null) {
-        continue;
-      }
-      TypeSymbol tsym = annotationType.tsym;
-      if (EXEMPTING_VARIABLE_ANNOTATIONS.contains(tsym.getQualifiedName().toString())) {
-        return true;
-      }
-    }
-    return false;
+    return parent instanceof EnhancedForLoopTree enhancedForLoopTree
+        && enhancedForLoopTree.getVariable() == tree;
   }
 
   private boolean exemptedByName(Name name) {
@@ -664,10 +634,8 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
           && exemptedFieldBySuperType(getType(variableTree), state)) {
         return;
       }
-      super.visitVariable(variableTree, null);
       // Return if the element is exempted by an annotation.
-      if (exemptedByAnnotation(variableTree.getModifiers().getAnnotations())
-          || shouldKeep(variableTree)) {
+      if (wellKnownKeep.shouldKeep(variableTree)) {
         return;
       }
       switch (symbol.getKind()) {
@@ -682,6 +650,14 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
           unusedElements.put(symbol, getCurrentPath());
           usageSites.put(symbol, getCurrentPath());
         }
+        case BINDING_VARIABLE -> {
+          if (parent instanceof BindingPatternTree
+              && getCurrentPath().getParentPath().getParentPath().getLeaf()
+                  instanceof InstanceOfTree) {
+            unusedElements.put(symbol, getCurrentPath());
+            usageSites.put(symbol, getCurrentPath());
+          }
+        }
         case PARAMETER -> {
           // ignore the receiver parameter
           if (variableTree.getName().contentEquals("this")) {
@@ -731,8 +707,8 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
       checkArgument(sym.getKind() == ElementKind.PARAMETER);
       Symbol enclosingMethod = sym.owner;
 
-      if (!(enclosingMethod instanceof MethodSymbol)
-          || isAbstract((MethodSymbol) enclosingMethod)
+      if (!(enclosingMethod instanceof MethodSymbol methodSymbol)
+          || isAbstract(methodSymbol)
           || superMethodsToOverrides.containsKey(enclosingMethod)) {
         return false;
       }
@@ -748,7 +724,7 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
       }
 
       if (enclosingMethod.owner instanceof ClassSymbol
-          && !isAbstract((MethodSymbol) enclosingMethod)
+          && !isAbstract(methodSymbol)
           && FUNCTIONAL_INTERFACE_TYPES_TO_CHECK.stream()
               .map(state::getTypeFromString)
               .anyMatch(
@@ -835,7 +811,7 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
 
     private boolean isInExpressionStatementTree() {
       Tree parent = getCurrentPath().getParentPath().getLeaf();
-      return parent != null && parent.getKind() == Kind.EXPRESSION_STATEMENT;
+      return parent instanceof ExpressionStatementTree;
     }
 
     private boolean isUsed(@Nullable Symbol symbol) {
@@ -917,7 +893,7 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
       if (!(parent instanceof StatementTree)) {
         return;
       }
-      if (tree.getVariable().getKind() != Kind.IDENTIFIER) {
+      if (!(tree.getVariable() instanceof IdentifierTree)) {
         return;
       }
       if (ASTHelpers.findEnclosingNode(getCurrentPath(), ForLoopTree.class) != null) {
@@ -962,8 +938,8 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
       if (assignmentSite.getParentPath().getLeaf() instanceof EnhancedForLoopTree) {
         return Iterables.size(assignmentSite) + 1;
       }
-      if (assignmentSite.getLeaf() instanceof VariableTree) {
-        VarSymbol symbol = getSymbol((VariableTree) assignmentSite.getLeaf());
+      if (assignmentSite.getLeaf() instanceof VariableTree variableTree) {
+        VarSymbol symbol = getSymbol(variableTree);
         if (symbol.getKind() == ElementKind.PARAMETER) {
           return Iterables.size(assignmentSite) + 1;
         }
@@ -1067,32 +1043,26 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
     }
   }
 
-  @AutoValue
-  abstract static class UnusedSpec {
-    /** {@link Symbol} of the unused element. */
-    abstract Symbol symbol();
-
-    /** {@link VariableTree} or {@link AssignmentTree} for the original assignment site. */
-    abstract TreePath assignmentPath();
-
-    /**
-     * All the usage sites of this variable that we claim are unused (including the initial
-     * declaration/assignment).
-     */
-    abstract ImmutableList<TreePath> usageSites();
-
-    /**
-     * If this usage chain was terminated by an unconditional reassignment, the corresponding {@link
-     * AssignmentTree}.
-     */
-    abstract Optional<AssignmentTree> terminatingAssignment();
-
+  /**
+   * @param symbol {@link Symbol} of the unused element.
+   * @param assignmentPath {@link VariableTree} or {@link AssignmentTree} for the original
+   *     assignment site.
+   * @param usageSites All the usage sites of this variable that we claim are unused (including the
+   *     initial declaration/assignment).
+   * @param terminatingAssignment If this usage chain was terminated by an unconditional
+   *     reassignment, the corresponding {@link AssignmentTree}.
+   */
+  private record UnusedSpec(
+      Symbol symbol,
+      TreePath assignmentPath,
+      ImmutableList<TreePath> usageSites,
+      Optional<AssignmentTree> terminatingAssignment) {
     private static UnusedSpec of(
         Symbol symbol,
         TreePath assignmentPath,
         Iterable<TreePath> treePaths,
         @Nullable AssignmentTree assignmentTree) {
-      return new AutoValue_UnusedVariable_UnusedSpec(
+      return new UnusedSpec(
           symbol,
           assignmentPath,
           ImmutableList.copyOf(treePaths),
