@@ -75,12 +75,16 @@ public final class ConstantExpressions {
   private final Supplier<ThreadSafety> threadSafety;
 
   @Inject
-  ConstantExpressions(WellKnownMutability wellKnownMutability) {
+  ConstantExpressions(WellKnownMutability wellKnownMutability, ErrorProneFlags flags) {
+    boolean considerAllMethodsPure =
+        flags.getBoolean("ConstantExpressions:ConsiderAllMethodsPure").orElse(false);
     this.pureMethods =
-        anyOf(
-            basePureMethods,
-            instanceMethod()
-                .onDescendantOfAny(wellKnownMutability.getKnownImmutableClasses().keySet()));
+        considerAllMethodsPure
+            ? anyMethod()
+            : anyOf(
+                basePureMethods,
+                instanceMethod()
+                    .onDescendantOfAny(wellKnownMutability.getKnownImmutableClasses().keySet()));
     this.threadSafety =
         memoize(
             s ->
@@ -93,7 +97,7 @@ public final class ConstantExpressions {
   }
 
   public static ConstantExpressions fromFlags(ErrorProneFlags flags) {
-    return new ConstantExpressions(WellKnownMutability.fromFlags(flags));
+    return new ConstantExpressions(WellKnownMutability.fromFlags(flags), flags);
   }
 
   /** Represents sets of things known to be true and false if a boolean statement evaluated true. */
@@ -263,6 +267,16 @@ public final class ConstantExpressions {
     return symbolizeImmutableExpression(tree, state).map(ConstantExpression::pureMethod);
   }
 
+  /** Returns whether {@code aTree} and {@code bTree} seem to correspond to the same expression. */
+  public boolean isSame(ExpressionTree aTree, ExpressionTree bTree, VisitorState state) {
+    var a = constantExpression(aTree, state);
+    if (a.isEmpty()) {
+      return false;
+    }
+    var b = constantExpression(bTree, state);
+    return b.isPresent() && a.get().equals(b.get());
+  }
+
   /** Represents a binary equals call on two constant expressions. */
   @AutoValue
   public abstract static class ConstantEquals {
@@ -272,10 +286,9 @@ public final class ConstantExpressions {
 
     @Override
     public final boolean equals(@Nullable Object other) {
-      if (!(other instanceof ConstantEquals)) {
+      if (!(other instanceof ConstantEquals that)) {
         return false;
       }
-      ConstantEquals that = (ConstantEquals) other;
       return (lhs().equals(that.lhs()) && rhs().equals(that.rhs()))
           || (lhs().equals(that.rhs()) && rhs().equals(that.lhs()));
     }
@@ -359,9 +372,10 @@ public final class ConstantExpressions {
     if (isPureIdentifier(tree)) {
       return Optional.of(
           PureMethodInvocation.of(getSymbol(tree), ImmutableList.of(), receiverConstant));
-    } else if (tree instanceof MethodInvocationTree && pureMethods.matches(tree, state)) {
+    } else if (tree instanceof MethodInvocationTree methodInvocationTree
+        && pureMethods.matches(tree, state)) {
       ImmutableList.Builder<ConstantExpression> arguments = ImmutableList.builder();
-      for (ExpressionTree argument : ((MethodInvocationTree) tree).getArguments()) {
+      for (ExpressionTree argument : methodInvocationTree.getArguments()) {
         Optional<ConstantExpression> argumentConstant = constantExpression(argument, state);
         if (argumentConstant.isEmpty()) {
           return Optional.empty();
@@ -494,7 +508,7 @@ public final class ConstantExpressions {
           allOf(
               instanceEqualsInvocation(),
               (t, s) -> {
-                if (!(t instanceof MethodInvocationTree)) {
+                if (!(t instanceof MethodInvocationTree methodInvocationTree)) {
                   return false;
                 }
                 ExpressionTree receiver = getReceiver(t);
@@ -502,16 +516,15 @@ public final class ConstantExpressions {
                   return false;
                 }
                 return typeIsImmutable(getType(receiver), s)
-                    && typeIsImmutable(
-                        getType(((MethodInvocationTree) t).getArguments().get(0)), s);
+                    && typeIsImmutable(getType(methodInvocationTree.getArguments().get(0)), s);
               }),
           allOf(
               staticEqualsInvocation(),
               (t, s) -> {
-                if (!(t instanceof MethodInvocationTree)) {
+                if (!(t instanceof MethodInvocationTree methodInvocationTree)) {
                   return false;
                 }
-                List<? extends ExpressionTree> args = ((MethodInvocationTree) t).getArguments();
+                List<? extends ExpressionTree> args = methodInvocationTree.getArguments();
                 return typeIsImmutable(getType(args.get(0)), s)
                     && typeIsImmutable(getType(args.get(1)), s);
               }));
