@@ -143,6 +143,7 @@ import com.google.errorprone.bugpatterns.ErroneousBitwiseExpression;
 import com.google.errorprone.bugpatterns.ErroneousThreadPoolConstructorChecker;
 import com.google.errorprone.bugpatterns.ExpectedExceptionChecker;
 import com.google.errorprone.bugpatterns.ExpensiveLenientFormatString;
+import com.google.errorprone.bugpatterns.ExplicitArrayForVarargs;
 import com.google.errorprone.bugpatterns.ExtendingJUnitAssert;
 import com.google.errorprone.bugpatterns.ExtendsAutoValue;
 import com.google.errorprone.bugpatterns.FallThrough;
@@ -373,6 +374,7 @@ import com.google.errorprone.bugpatterns.StringCaseLocaleUsage;
 import com.google.errorprone.bugpatterns.StringCharset;
 import com.google.errorprone.bugpatterns.StringConcatToTextBlock;
 import com.google.errorprone.bugpatterns.StringFormatWithLiteral;
+import com.google.errorprone.bugpatterns.StringJoin;
 import com.google.errorprone.bugpatterns.StringSplitter;
 import com.google.errorprone.bugpatterns.StronglyTypeByteString;
 import com.google.errorprone.bugpatterns.SubstringOfZero;
@@ -389,6 +391,7 @@ import com.google.errorprone.bugpatterns.SystemOut;
 import com.google.errorprone.bugpatterns.TestExceptionChecker;
 import com.google.errorprone.bugpatterns.TestParametersNotInitialized;
 import com.google.errorprone.bugpatterns.TheoryButNoTheories;
+import com.google.errorprone.bugpatterns.ThreadBuilderNameWithPlaceholder;
 import com.google.errorprone.bugpatterns.ThreadJoinLoop;
 import com.google.errorprone.bugpatterns.ThreadLocalUsage;
 import com.google.errorprone.bugpatterns.ThreeLetterTimeZoneID;
@@ -503,6 +506,7 @@ import com.google.errorprone.bugpatterns.flogger.FloggerLogString;
 import com.google.errorprone.bugpatterns.flogger.FloggerLogVarargs;
 import com.google.errorprone.bugpatterns.flogger.FloggerLogWithCause;
 import com.google.errorprone.bugpatterns.flogger.FloggerMessageFormat;
+import com.google.errorprone.bugpatterns.flogger.FloggerPerWithoutRateLimit;
 import com.google.errorprone.bugpatterns.flogger.FloggerRedundantIsEnabled;
 import com.google.errorprone.bugpatterns.flogger.FloggerRequiredModifiers;
 import com.google.errorprone.bugpatterns.flogger.FloggerSplitLogStatement;
@@ -845,11 +849,13 @@ public class BuiltInCheckerSuppliers {
           SizeGreaterThanOrEqualsZero.class,
           StreamToString.class,
           StringBuilderInitWithChar.class,
+          StringJoin.class,
           SubstringOfZero.class,
           SuppressWarningsDeprecated.class,
           TemporalAccessorGetChronoField.class,
           TestParametersNotInitialized.class,
           TheoryButNoTheories.class,
+          ThreadBuilderNameWithPlaceholder.class,
           ThrowIfUncheckedKnownChecked.class,
           ThrowNull.class,
           TreeToString.class,
@@ -956,6 +962,7 @@ public class BuiltInCheckerSuppliers {
           FloatingPointAssertionWithinEpsilon.class,
           FloatingPointLiteralPrecision.class,
           FloggerArgumentToString.class,
+          FloggerPerWithoutRateLimit.class,
           FloggerStringConcatenation.class,
           FragmentInjection.class,
           FragmentNotInstantiable.class,
@@ -1206,6 +1213,7 @@ public class BuiltInCheckerSuppliers {
           EqualsBrokenForNull.class,
           EqualsMissingNullable.class,
           ExpectedExceptionChecker.class,
+          ExplicitArrayForVarargs.class,
           ExtendsAutoValue.class,
           FieldCanBeFinal.class,
           FieldCanBeLocal.class,
